package com.examples;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.sql.SQLException;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.util.Properties;

public class MySQLDAO extends DAOBase
{
	private  final  String  DRIVER_MySQL   = "com.mysql.jdbc.Driver";
	private  final  String  URL_mysql      = "jdbc:mysql://%s:%d/%s";

	private  final  String  URL_host       = "localhost"            ;
	private  final  int     PORT_mysql     = 3306                   ;
	public   final  String  DATABASE       = "test"                 ;

	private  final  String  login          = "...";
	private  final  String  password       = "...";

	private  final  String  TABLE_blobs = "CREATE TABLE blobs( "           +
                                                 "id    INT PRIMARY KEY, " +
                                                 "data  BLOB NULL)"        ;
	private  final  String  TABLE_files  = "CREATE TABLE files( "          +
                                                 "id    INT PRIMARY KEY, " +
                                                 "data  TEXT NULL) "       +
                                                 "ENGINE = INNODB "        +
                                                 "CHARACTER SET utf8 "     +
                                                 "COLLATE utf8_general_ci;";
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public MySQLDAO()
	{
		createConnection();
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	@Override
	public void createConnection()
	{
		try {
			// Регистрация драйвера
			Class.forName(DRIVER_MySQL).newInstance();
			// Определение свойств подключения Connection
			Properties properties = new Properties();
			properties.setProperty("password"         , password);
			properties.setProperty("user"             , login   );
			properties.setProperty("useUnicode"       , "true"  );
			properties.setProperty("characterEncoding", "utf8"  );
			
			String url = String.format(URL_mysql, URL_host, PORT_mysql, DATABASE);
			connection = DriverManager.getConnection(url, properties);
			connection.setAutoCommit(false);
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		} catch (ClassNotFoundException e) {
		} catch (SQLException           e) {
			connection = null;
		}
	};
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	@Override
	public boolean createTable(final String table)
	{
		if (table.equalsIgnoreCase("blobs"))
			return execSQL (TABLE_blobs);
		else if (table.equalsIgnoreCase("files"))
			return execSQL (TABLE_files);
		else
			return false;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи содержимого файла в поле CLOB 
	 * @param table наименование таблицы
	 * @param field наименование поля BLOB в таблице
	 * @param pk Primary Key
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return результат записи (true - OK, false - failed)
	 */
	@Override
	public boolean writeClob(final String table, final String field, 
                             final String pk, final int id,
                             final String fpath)
	{
		String sql = String.format(INSERT_clob, table, pk, id);
		boolean result = execSQL (sql);
		if (result) {
			PreparedStatement ps = null;
	        File file = new File(fpath);
	        
	        try {
				FileInputStream   fis = new FileInputStream  (file);

		        sql = String.format(UPDATE_clob, table, field, pk);
				ps = connection.prepareStatement(sql);
				ps.setAsciiStream (1, fis, (int) file.length());
				ps.setInt         (2, id);
				ps.executeUpdate();

				connection.commit();
				ps.close();
			} catch (FileNotFoundException e) {
				result = false;
				e.printStackTrace();
			} catch (SQLException e) {
				result = false;
				e.printStackTrace();
			}
		}
		return result;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
